package de.ozgcloud.apilib.vorgang.grpc;

import static org.assertj.core.api.Assertions.*;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;
import org.mockito.Spy;

import de.ozgcloud.apilib.vorgang.OzgCloudVorgangEingangTestFactory;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangHeaderTestFactory;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangIdMapper;
import de.ozgcloud.apilib.vorgang.OzgCloudVorgangTestFactory;

class OzgCloudVorgangMapperTest {

	@InjectMocks
	private OzgCloudVorgangMapper mapper = Mappers.getMapper(OzgCloudVorgangMapper.class);

	@Spy
	private OzgCloudEingangMapper eingangMapper = Mappers.getMapper(OzgCloudEingangMapper.class);
	@Spy
	private OzgCloudVorgangIdMapper idMapper = Mappers.getMapper(OzgCloudVorgangIdMapper.class);

	@Nested
	class TestFromGrpc {
		@Test
		void shouldMapVorgangStub() {
			var mapped = mapper.fromGrpc(GrpcVorgangWithEingangTestFactory.create());

			assertThat(mapped).usingRecursiveComparison()
					.ignoringFields("eingangs", "header")
					.isEqualTo(OzgCloudVorgangTestFactory.create());
		}

		@Test
		void shouldMapHeader() {
			var mapped = mapper.fromGrpc(GrpcVorgangWithEingangTestFactory.create());

			assertThat(mapped.getHeader()).usingRecursiveComparison()
					.isEqualTo(OzgCloudVorgangHeaderTestFactory.create());
		}

		@Test
		void shouldMapEingang() {
			var mapped = mapper.fromGrpc(GrpcVorgangWithEingangTestFactory.create());

			assertThat(mapped.getEingangs()).hasSize(1).first()
					.usingRecursiveComparison().ignoringFields("attachments")
					.isEqualTo(OzgCloudVorgangEingangTestFactory.create());
		}
	}

}
